#!/bin/bash
#

# build_wmap.sh  <product_directory> <e4> <debug|opt|prof> [tar]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e4> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

function die()
{
  echo "$1"; exit 2
}

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
    if [ -z ${basequal} ]
    then
        echo "ERROR: please specify the qualifiers"
        usage
        exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	      qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	      qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	      qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
        echo "ERROR: please specify debug, opt, or prof"
        usage
        exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    patchdir=${product_dir}/${package}/${pkgver}/patch
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

select_gcc()
{
    if [ "${basequal}" = "e4" ]
    then
       gccdotver=4.8
       gccver=v4_8_1
    else
      echo "ERROR: unrecognized qualifier ${basequal}"
      usage
      exit 1
    fi
}

setup_gcc()
{
    setup gcc ${gccver}
    gccshortver=`gcc --version | head -1 | cut -f3 -d" " | cut -f1-2 -d"."`
    gccfullver=`gcc --version | head -1 | cut -f3 -d" "`
    if [ ${gccshortver} = ${gccdotver} ]
    then
      echo " OK - gcc version is ${gccfullver}"
    else
      echo "ERROR: wrong gcc version ${gccfullver}"
      exit 1
    fi
}

setup_cfitsio()
{
    # basequal carries the C++ or Fortran version info, and is not relevant for cfitsio
    # extraqual carries the debug/opt/prof choice
    case $extraqual in
        opt|debug|prof)
            setup -B cfitsio $cfitsio_version -q +${extraqual} || die "Setup of cfitsio failed"
            ;;
        *)
            die "Unexpected value of extraqual: ${extraqual}"
    esac
}

setup_lapack()
{
    # basequal carries the C++ or Fortran version info, and must match
    # extraqual carries debug/opt/prof choice, and is irrelevant for lapack: we always use 'prof'.
    setup -B lapack $lapack_version -q +${basequal}:+prof || die "Setup of lapack failed"
}

setup_wmapdata()
{
    # wmapdata is unqualified and unflavored
    setup -B wmapdata ${wmapdata_version} || die "Setup of wmapdata failed"
}

declare_product()
{
    echo "Declaring ${package} ${pkgver} for ${flvr} and ${fullqual}"
    if [ ${OS1} = "Darwin" ]
    then
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -2 -m ${package}.table -q ${fullqual} -z ${product_dir}
    else 
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -4 -m ${package}.table -q ${fullqual} -z ${product_dir}
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

# WARNING: The source distribution comes with in a tarball that does not
# match common naming conventions, and so is hard-coded into
# 'pkgtarname' below. Note that this does NOT have the .tar.gz included.
package=wmap
pkgver=v5_00b
ssibuildshims_version=v0_06_03
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=wmap_likelihood_sw_v5

# -------------------------------------------------------------------
# Versions of things we depend upon
# This information is repeated in the table file.
# -------------------------------------------------------------------
cfitsio_version=v3_35_0
lapack_version=v3_4_2
wmapdata_version=v5_00

get_this_dir
get_ssibuildshims
set_qualifiers
get_platform_name
select_gcc

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    cxxflg="-O3 -g -DNDEBUG"
elif [ "${extraqual}" = "debug" ]
then
    cxxflg="-g -O0"
elif [ "${extraqual}" = "prof" ]
then
    cxxflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
fi

fflg="${cxxflg} -ffree-line-length-none -fPIC -J${pkgdir}/include"

setup_gcc
setup_lapack
setup_cfitsio
setup_wmapdata

mkdir -p ${sourcedir} 
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

if [ ${OS1} = "Darwin" ]
then
   PATH=`dropit /usr/bin/get-cert $PATH`
fi

set -x

cd ${sourcedir}
tar xf ${tardir}/${pkgtarname}.tar.gz || die "Failed to untar ${pkgtarname}.tar.gz"
cd ${product_dir}/${package}/${pkgver}
patch -p1 < ${patchdir}/wmap.patch || die "Failed to apply patch"

mkdir -p ${pkgdir}/lib     || die "Failed to create ${pkgdir}/lib"
mkdir -p ${pkgdir}/include || die "Failed to create ${pkgdir}/include"

# wmap does an in-source build.
set +x
cd ${sourcedir}/wmap_likelihood_v5 || die "Unable to cd to ${sourcedir}/${pkgtarname}"

if [ ! -f Makefile ]
then
    echo "ERROR: no Makefile in $PWD"
    exit 5
fi

set -x

FFLAGS="${fflg}" make || die "make failed"
LD_LIBRARY_PATH=.:$LD_LIBRARY_PATH FFLAGS="${fflg}" make check || die "make check failed"

# Move library to final location
mv libwmap9.so ${pkgdir}/lib || die "Failed to copy library to final location"

# Remove build debris
make clean
rm ${sourcedir}/wmap_likelihood_v5/test

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

declare_product

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "wmap is installed at ${WMAP_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
